import type { Id } from "../Id";
import type { GameData } from "../GameData";
import type { State } from "../State";
import type { GamePlayer } from "$lib/GamePlayer";

export class Game implements GameData {
	players: GamePlayer[];
	isStarted: boolean;
	state: State;

	constructor() {
		this.players = [];
		this.isStarted = false;
		this.state = {};
	}

	static from(data: GameData) {
		const game = new Game();
		game.players = data.players;
		game.isStarted = data.isStarted;
		game.state = data.state;

		return game;
	}

	addPlayer(player: GamePlayer) {
		this.players.push(player);
	}

	setPlayerReady({ id: playerId, isReady }: GamePlayer) {
		const player = this.players.find(({ id }) => playerId === id);

		if (!player) return null;

		player.isReady = isReady;

		return player;
	}

	start() {
		if (this.isStarted) throw new Error("game is already started");

		this.isStarted = true;
	}
}
