import { describe, it } from "vitest";
import { Game } from "$lib/server/Game";
import { ok, throws } from "node:assert/strict";
import { createId } from "$lib/Id";
import { equal, deepEqual } from "node:assert";

describe("Game", () => {
	const idString = createId();
	describe("addPlayer", () => {
		it("should push a player id into the player array", () => {
			const game = new Game();
			const user = "Mr. User";

			equal(game.players.length, 0);

			game.addPlayer({ id: idString, username: user, isReady: false });
			equal(game.players.length, 1);
			deepEqual(game.players[0], { id: idString, username: user, isReady: false });
		});
	});

	describe("start", () => {
		it("start shoud start the game", () => {
			const game = new Game();
			game.isStarted = false;

			game.start();
			ok(game.isStarted);
		});

		it("start should throw if the game is already started", () => {
			const game = new Game();

			game.start();
			throws(() => game.start());
		});
	});
});
