import { describe, it } from "vitest";
import { isGameData } from "$lib/GameData";
import { equal, ok } from "node:assert/strict";
import { createId } from "$lib/Id";

describe("GameData", () => {
	const idString = createId();

	describe("isGameData", () => {
		it("rejects a malformed object", () => {
			let data: unknown = {
				players: [idString, idString],
				isStarted: false,
				state: {},
			};
			equal(isGameData(data), false);

			data = {
				players: [idString],
				isStarted: null,
				state: {},
			};
			equal(isGameData(data), false);

			data = {
				players: [{ username: "Mr. User", id: idString }],
				isStarted: false,
			};
			equal(isGameData(data), false);
		});

		it("rejects an object with a malformed players array", () => {
			let data: unknown = {
				players: [{ id: idString, username: "Mr. User" }],
				state: {},
				isStarted: false,
			};
			equal(isGameData(data), false);

			data = {
				players: [{ id: idString, isReady: false }],
				state: {},
				isStarted: false,
			};
			equal(isGameData(data), false);

			data = {
				players: [{ username: "Mr. User", isReady: false }],
				state: {},
				isStarted: false,
			};
			equal(isGameData(data), false);
		});

		it("rejects an object without a players property", () => {
			const data: unknown = {
				state: {},
				isStarted: false,
			};

			equal(isGameData(data), false);
		});

		it("rejects an object with extra properties", () => {
			const data: unknown = {
				players: [{ username: "Mr. User", id: idString, isReady: false }],
				isStarted: false,
				state: {},
				extra: true,
			};

			equal(isGameData(data), false);
		});

		it("should accept a proper GameData object", () => {
			const data: unknown = {
				players: [{ username: "Mr. User", id: idString, isReady: false }],
				state: {},
				isStarted: false,
			};

			ok(isGameData(data));
		});
	});
});
