import { isGameData, type GameData } from "$lib/GameData";
import { isListing } from "$lib/Listing";
import { Game } from "$lib/server/Game";
import { updateListing } from "$lib/server/modifyListing";
import {
	readListingById,
	ServerCollections,
	writeUpdatedListing,
} from "$lib/server/mongo";
import { getBody, getParam, ResourceId } from "$lib/server/requestTools";
import {
	badRequestResponse,
	conflictResponse,
	notFoundResponse,
	singleResponse,
} from "$lib/server/responseBodies";
import type { RequestHandler } from "@sveltejs/kit";

export const GET: RequestHandler = async ({ params }): Promise<Response> => {
	const id = getParam(params, ResourceId.Game);

	if (!id) {
		return badRequestResponse("missing gameid parameter");
	}

	const game = await readListingById(ServerCollections.Games, id, isListing<GameData>);

	if (!game) {
		return notFoundResponse();
	}

	return singleResponse(game);
};

export const PUT: RequestHandler = async ({ request, params }): Promise<Response> => {
	const id = getParam(params, ResourceId.Game);

	if (!id) {
		return badRequestResponse("missing gameid parameter");
	}

	const gameListing = await readListingById(
		ServerCollections.Games,
		id,
		isListing<GameData>,
	);

	if (!gameListing) {
		return notFoundResponse();
	}

	const game = Game.from(gameListing.data);
	const body = await getBody(request, isGameData);

	if (!body) {
		return badRequestResponse("missing game data in body");
	}

	if (!game.isStarted) {
		if (body.isStarted) {
			game.start();
		}
	} else if (game.isStarted !== body.isStarted) {
		return conflictResponse();
	}

	await writeUpdatedListing(ServerCollections.Games, updateListing(gameListing, game));

	return singleResponse(gameListing);
};
